
# q-expansion of newform 1408.2.a.n, downloaded from the LMFDB on 12 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1408
weight = 2
poly_data = [-2, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0, 0], [1, 1], [1, 2], [0, 1], [-1, 0], [0, -3], [-6, 1], [2, -1], [7, -1], [8, 0], [1, 5], [-3, -2], [0, -5], [2, -2], [4, -6], [0, 2], [1, -9], [0, 4], [3, 9], [1, -5], [-8, -5], [2, -2], [14, -2], [-1, -2], [-9, 0], [-6, -7], [2, 4], [-4, 5], [-2, -8], [-1, 6], [-2, -7], [0, 5], [9, 2], [12, 3], [-14, 1], [2, 2], [7, -6], [4, -12], [-10, -3], [18, 0], [-15, -7], [7, 4], [-7, -3], [0, -4], [2, -4], [0, 8], [-8, 4], [-9, -3], [16, 9], [1, 12], [0, -8], [18, -2], [16, -8], [-3, 7], [-12, -10], [-4, 19], [8, 4], [-18, -1], [2, 12], [-10, 12], [-8, 4], [12, 11], [18, 5], [24, -6], [9, 2], [11, -6], [-11, -13], [-6, -9], [10, -5], [-22, 1], [-3, 14], [-10, -7], [1, -7], [-6, 11], [-21, 11], [-3, -1], [9, -12], [16, -2], [4, -18], [2, 16], [16, 14], [16, -6], [0, 11], [13, -8], [-4, -8], [1, 7], [3, 10], [-20, 0], [4, 12], [7, 9], [17, -13], [-2, 11], [13, -21], [4, -16], [4, 8], [18, -6], [33, 4], [11, 4], [2, -25], [-16, -5], [12, -4], [-6, 9], [4, -16], [4, 16], [-12, 4], [-7, -10], [2, 8], [-8, 24], [0, -4], [-22, -1], [2, -14], [20, 0], [-12, 20], [-17, -9], [-7, -3], [1, -12], [-19, -11], [1, 9], [33, 4], [18, 18], [-11, -8], [26, 0], [-10, -3], [36, -2], [17, -15], [10, -9], [41, 0], [37, 11], [31, -3], [12, -8], [22, -10], [-16, 0], [11, -3], [20, -20], [-24, -9], [12, -1], [10, -16], [28, 16], [5, -16], [-28, -5], [-6, 6], [-26, 16], [-13, 5], [14, 5], [-25, 22], [9, 19], [22, 8], [32, 8], [15, -21], [16, -24], [0, -23], [-5, 2], [18, 8], [38, 6], [-4, -12], [-32, -14], [-10, 18], [-12, 14], [-36, -11], [22, -8], [-45, -1], [-14, 25], [-8, -12], [5, -9], [-27, 2], [37, -9], [-40, -8], [-26, -7], [22, 0], [17, 20], [14, 2], [-24, -8], [-34, 3], [0, 23], [41, -1], [25, -16], [-24, 23], [-11, 18], [24, 0], [-24, 12], [-32, 16], [-30, -6], [-13, -6], [-10, -12], [-23, 3], [-32, 2], [4, 19], [-44, -14], [-6, -15], [10, 6], [-15, 2], [-12, -7], [-39, 15], [-54, -8], [26, 13], [-21, 2], [-2, -1], [-15, 16], [-14, 4], [48, 13], [-12, 35], [-10, 30], [6, 8], [28, 28], [-21, 17], [25, 6], [35, 5], [-48, 12], [16, -4], [28, -10], [-40, 12], [19, -22], [-17, 3], [14, -16], [-4, 5], [31, 0], [-2, 31], [-16, 0], [2, 20], [-9, -8], [-36, 15], [-18, -17], [7, 4], [27, 31], [18, -31], [-42, -16], [-24, -20], [6, -36], [46, 13], [-54, -1], [-15, 10], [-26, -1], [-10, -2], [4, -40], [15, 27], [-14, 30], [-41, -14], [-20, 9], [15, 7], [27, 27], [19, 21], [44, 16], [8, -18], [-17, -22], [8, -44], [0, 36], [24, -18], [52, 4], [-34, -18], [18, -2], [60, -8], [14, 7], [3, -43], [-20, 34], [-2, -51], [-28, -8], [-10, -16], [32, 23], [1, -2], [-26, 3], [-60, -4], [-2, 21], [42, 21], [10, 27], [-28, -6], [36, 10], [19, -34], [13, 20], [6, -10], [-2, -28], [33, 8], [-27, -25], [4, -14], [18, 17], [-24, 5], [-3, 25], [-17, 19], [-22, -4], [-24, 19], [14, 15], [26, 3], [43, 27], [46, -5], [6, 0], [24, 20], [-63, 11], [12, 0], [-26, 5], [-1, -27], [-54, 15], [-27, -6], [16, -26], [12, -16], [-48, 11], [-18, 21], [36, -3], [-27, 3], [47, -10], [4, 45], [22, -6], [-14, -9], [84, 3], [-12, -40], [20, 28], [-29, -9], [-33, 18], [-3, -13], [5, -38], [-16, -2], [4, 19], [16, -4], [-24, -40], [18, 24], [61, 15], [-28, -8], [-34, -17], [-7, 51], [54, 2], [28, 6], [52, 21], [-50, 9], [-9, 44], [22, -9], [-13, -5], [22, -12], [-13, 24], [-63, 11], [15, 11], [2, 9], [20, -44], [-62, 0], [-48, 14], [12, -43], [-20, 12], [8, 12], [67, -5], [7, 0], [40, 24], [-71, 6], [12, -3], [5, 2], [1, -36], [0, -21], [21, -23], [13, 26], [-2, -14], [-3, -4], [-33, -31], [-58, -25], [-25, -24], [10, 1], [-5, -48], [-28, 24], [16, -24], [58, -9], [-14, 0], [-5, -37], [-22, -38], [14, 32], [-41, 31], [10, 44], [-22, -3], [-39, 11], [40, -31], [-7, -49], [27, 4], [8, 13], [66, 6], [20, -33], [-41, 15], [18, -20], [4, -35], [16, 21], [-38, 14], [43, -8], [34, -24], [52, -27], [-26, 24], [46, 28], [-22, 19], [-56, -10], [-33, 26], [-40, -21], [-2, -33], [40, 4], [31, 3], [-4, 66], [-39, 40], [10, 25], [9, 25], [-13, -56], [15, -44], [-41, 39], [-47, 13], [-35, 19], [-4, -63], [-4, -7], [28, -18], [0, -14], [-58, -20], [2, -7], [-17, -4], [8, -8], [-44, 10], [-32, 28], [8, -45], [-22, -8], [80, 5], [-28, -36], [47, 11], [26, -56], [-6, 19], [36, -36], [52, -1], [-18, 41], [-1, -12], [-36, 24], [-61, 9], [66, 16], [-10, 2], [-65, 20], [-32, -36], [2, -4], [-64, 15], [-40, -20], [-43, -4], [-7, -7], [-6, 37], [-3, -23], [8, -60]]
